DECLARE FUNCTION fAsciiHexToBinary% (StringToConvert AS STRING)
DECLARE FUNCTION fBinaryToAsciiHex$ (ByteToConvert AS INTEGER)
DECLARE SUB ConfigurePort ()
DECLARE SUB NodeActivities ()
DECLARE SUB onebytedelay (NumberOfBytes AS INTEGER)
DECLARE SUB ReadMessage ()
DECLARE SUB SendAck ()
DECLARE SUB SendReply ()
DECLARE SUB ShortDelay ()

DIM SHARED BitRate AS INTEGER
DIM SHARED BitRateString AS STRING
DIM SHARED Buffer AS STRING
DIM SHARED ComPortAddress AS INTEGER
DIM SHARED ComPortAddressString AS STRING
DIM SHARED ComPortNumber AS INTEGER
DIM SHARED ComPortNumberString AS STRING
DIM SHARED DataIn1 AS INTEGER
DIM SHARED DataIn2 AS INTEGER
DIM SHARED DataOut1 AS INTEGER
DIM SHARED DataOut2 AS INTEGER
DIM SHARED FileExists AS INTEGER
DIM SHARED LSRAddress AS INTEGER
DIM SHARED MessageOut AS STRING
DIM SHARED MSR AS INTEGER
DIM SHARED MSRAddress AS INTEGER
DIM SHARED TimedOut AS INTEGER

CONST NodeAddress = &H68
CONST True = -1
CONST False = NOT True
ON ERROR GOTO ErrorHandler

'Slave node
'On detecting a received byte at the serial port,
'see if it matches the node address.
'If yes, respond with an Acknowledge,
'wait to receive four ASCII hex bytes,
'then send the node address & four ASCII hex bytes to the master.
'If the received byte doesn't match the node address, ignore it.

CALL ConfigurePort

'Initialize the selected COM port & set communications parameters:
'bit rate, no parity, 8 data bits, 1 stop bit, no handshaking.
OPEN "COM" + ComPortNumberString + ": " + BitRateString + ",n,8,1,bin,rs,cs0,ds0,cd0" FOR RANDOM AS #1
'Additional UART registers used:
'Modem status register:
MSRAddress = ComPortAddress + 4
'Line status register:
LSRAddress = ComPortAddress + 5

'Clear any error flags
LSR = INP(LSRAddress)
'Clear any unread data
DataRead = INP(ComPortAddress)
MSR = INP(MSRAddress)
'Set RTS=1 to enable the network receiver
MSR = MSR OR 2
OUT (MSRAddress), MSR
CALL ShortDelay
'Basic checks the COM port for received data
'after executing each statement.
'When data has been received, see if the node number matches.
'If yes, send Acknowledge and wait for data.
'If no, ignore.

ON COM(ComPortNumber) GOSUB ReadSerialPort
COM(ComPortNumber) ON
PRINT "waiting for data"
WHILE 1 = 1 'endless loop
'Place any other node activities here.
CALL NodeActivities
WEND

END

ReadSerialPort: 
'Read a byte from the serial-port buffer
'Ignore any new Com events.
'(If Com port is On, QB issues a Com Stop until return from subroutine.)
COM(ComPort) OFF
'Read one byte.
Buffer = INPUT$(LOC(1), #1)
'See if the byte matches the node address
IF Buffer = CHR$(NodeAddress) THEN
	PRINT "Byte received = ", HEX$(ASC(Buffer)) + "h"
	CALL SendAck
	CALL ReadMessage
	CALL SendReply
END IF
'Re-enable event trapping
COM(ComPort) ON
RETURN

TimeOut:
TimedOut = True
TIMER OFF
PRINT "Timeout"
RETURN

ErrorHandler:
SELECT CASE ERR
CASE 53
		FileExists = False
		RESUME NEXT
	CASE ELSE
		
	RESUME NEXT
END SELECT

SUB ActOnReceivedMessage
'Perform any tasks requested by the network message.
'This test routine sets each of the two bytes
'to send back to the master equal to a byte received + 1.
DataOut1 = DataIn1 + 1
IF DataOut1 > 255 THEN DataOut1 = 0
DataOut2 = DataIn2 + 1
IF DataOut2 > 255 THEN DataOut2 = 0
END SUB

SUB ConfigurePort
'Set the COM port number, address, and bit rate.
'Previous settings are saved in a file.
CONST SettingsFile = 2
DIM AcceptSettings AS STRING
DIM SettingsFileName AS STRING
SettingsFileName = "485nwcfg.dat"
'Open or create the file to store port settings.
FileExists = True
OPEN SettingsFileName FOR INPUT AS #SettingsFile
IF FileExists = True THEN
	INPUT #SettingsFile, ComPortNumberString, ComPortAddressString, BitRateString
	CLOSE #SettingsFile
END IF
IF VAL(ComPortNumberString) < 1 THEN
	ComPortNumberString = "1"
 END IF
IF VAL(ComPortAddressString) < 1 THEN
	ComPortAddressString = "3F8"
END IF
IF VAL(BitRateString) < 100 THEN
	BitRateString = "9600"
END IF
'Display the settings and allow user to change them.
CLS
PRINT "Current COM port settings are:"
PRINT "1  ", "Port Number = ", ComPortNumberString
PRINT "2  ", "Base Address = ", ComPortAddressString
PRINT "3  ", "BitRate = ", BitRateString
AcceptSettings = "Y"
INPUT "OK? (Y/N) Y", AcceptSettings
SELECT CASE AcceptSettings
	CASE "N", "n"
		INPUT "Port Number:  ", ComPortNumberString
		INPUT "Base Address:  ", ComPortAddressString
		ComPortAddressString = UCASE$(ComPortAddressString)
		'If the user adds a trailing "H", remove it.
		SELECT CASE RIGHT$(ComPortAddressString, 1)
			CASE "H"
				ComPortAddressString = LEFT$(ComPortAddressString, LEN(ComPortAddressString) - 1)
			CASE ELSE
		END SELECT
		INPUT "BitRate:  ", BitRateString
	 CASE "q"
END SELECT

'Get the values of the strings.
ComPortNumber = VAL(ComPortNumberString)
ComPortAddress = VAL("&h" + ComPortAddressString)
BitRate = VAL(BitRateString)
CLOSE #2
IF AcceptSettings = "n" OR AcceptSettings = "N" THEN
	OPEN SettingsFileName FOR OUTPUT AS #SettingsFile
	WRITE #SettingsFile, ComPortNumberString, ComPortAddressString, BitRateString
END IF
CLOSE #SettingsFile

PRINT ComPortNumber, HEX$(ComPortAddress), BitRate
END SUB

FUNCTION fAsciiHexToBinary% (StringToConvert AS STRING)
'Converts a 2-byte ASCII hex string to its byte value
	fAsciiHexToBinary = VAL("&h" + StringToConvert)
END FUNCTION

FUNCTION fBinaryToAsciiHex$ (ByteToConvert AS INTEGER)
'Converts a binary value to a 2-character ASCII hex string.
DIM Result AS STRING
Result = HEX$(ByteToConvert)
IF LEN(Result) < 2 THEN
	DO
		Result = "0" + Result
	LOOP UNTIL LEN(Result) = 2
END IF
fBinaryToAsciiHex = Result
END FUNCTION

SUB NodeActivities
'Perform any other activities the node is responsible for.                                                                                                                                                                                                             'For testing, set dummy values to send:
DataOut1 = DataOut1 + 1
IF DataOut1 > 255 THEN DataOut1 = 0
DataOut2 = DataOut1 + 2
IF DataOut2 > 255 THEN DataOut2 = 0
END SUB

SUB onebytedelay (NumberOfBytes AS INTEGER)
'Used to allow time for bytes to transmit
'before disabling the transmitter.
DIM CurrentTime AS SINGLE
DIM DelayTime AS SINGLE
DIM StartTime AS SINGLE
'DelayTime in seconds.
DelayTime = 10 / BitRate
StartTime = TIMER
DO
CurrentTime = TIMER
'In case TIMER rolls over to 0:
IF CurrentTime < StartTime THEN CurrentTime = StartTime
LOOP UNTIL CurrentTime > StartTime + DelayTime
END SUB

SUB ReadMessage
'Read and display received bytes.
DIM Message AS STRING
DIM ByteNumber AS INTEGER
DIM WaitTime AS INTEGER
TimedOut = False
WaitTime = 4
Message = ""
'Enable timer event trapping
TIMER ON
'Give up after WaitTime.
ON TIMER(WaitTime) GOSUB TimeOut
FOR ByteNumber = 1 TO 4
	'Wait for a byte to arrive
	DO
		Buffer = INPUT$(LOC(1), #1)
	LOOP UNTIL ((Buffer <> "") OR (TimedOut = True))
	'Add it to the message
	Message = Message + Buffer
NEXT ByteNumber
TIMER OFF
TimedOut = False
'get the binary values of the ASCII hex bytes received.
DataIn1 = fAsciiHexToBinary(MID$(Message, 1, 2))
DataIn2 = fAsciiHexToBinary(MID$(Message, 3, 2))
'Display the results.
PRINT "Data bytes received: "; HEX$(DataIn1); "h, "; HEX$(DataIn2); "h"
END SUB

SUB SendAck
'Send acknowledge after receiving a byte that matches the node address.
'Toggle RTS (bit 1 of MSR) on to send ACK, then off when finished.
DIM I AS INTEGER
'Delay to be sure master's transmitter is off
CALL ShortDelay
MSR = INP(MSRAddress)
'Enable the driver (RTS=0)
MSR = MSR AND &HFD
OUT (MSRAddress), MSR
CALL ShortDelay
PRINT #1, CHR$(NodeAddress); 'send the node address

CALL onebytedelay(1)
PRINT "Sending ACK"
MSR = INP(MSRAddress)
'Enable the receiver (RTS=1)
MSR = MSR OR 2
OUT (MSRAddress), MSR
END SUB

SUB SendReply
'On receiving a message, reply by sending 0, followed by two data bytes.
DIM OutputString1 AS STRING
DIM OutputString2 AS STRING
'Toggle RTS (MSR bit 1) on to send reply, then off when finished.
'Wait to allow time for master to disable its transmitter.
CALL ShortDelay
MSR = INP(MSRAddress)
'Enable the driver (RTS=0)
MSR = MSR AND &HFD
OUT (MSRAddress), MSR
CALL onebytedelay(2)
CALL ShortDelay
'Be sure each hex string has 2 bytes, even if <10h
OutputString1 = fBinaryToAsciiHex(DataOut1)
OutputString2 = fBinaryToAsciiHex(DataOut2)
MessageOut = CHR$(NodeAddress) + OutputString1 + OutputString2
PRINT #1, MessageOut;
CALL onebytedelay(4)
PRINT "Data bytes sent: "; HEX$(NodeAddress); "h "; HEX$(DataOut1); "h, "; HEX$(DataOut2); "h"
MSR = INP(MSRAddress)
'Enable the receiver (RTS=1)
MSR = MSR OR 2
OUT (MSRAddress), MSR
END SUB

SUB ShortDelay
DIM CurrentTime AS SINGLE
DIM StartTime AS SINGLE
StartTime = TIMER
DO
	CurrentTime = TIMER
	'Just in case TIMER rolls over to 0:
	IF CurrentTime < StartTime THEN CurrentTime = StartTime + 1
LOOP UNTIL CurrentTime > StartTime + .2
END SUB

